/**
 * @file
 * Provides GridStack admin utilities for Layout Builder inline.
 *
 * Cheaper alt than full-blown class overrides via hook_element_plugin_alter
 * aside from the fact other modules might also race overriding that class.
 */

(function ($, Drupal, GridStyle, _win, _doc) {

  'use strict';

  var _dgs = 'data-gs-';
  var elBody = _doc.body;
  var cGsActionAjax = 'gridstack__action--ajax-styles';
  var cGsActionJs = 'gridstack__action--js-styles';
  var cLbLinkConfig = 'layout-builder__link--configure';
  var cLbHighligted = 'is-layout-builder-highlighted';
  var cGridstackAction = 'gridstack__action';
  var cGsRegionActive = 'is-gs-region-active';
  var cFormSubmitActive = 'form-submit--gs-active';
  var cLoading = 'is-b-loading';
  var cVisuallyHidden = 'visually-hidden';
  var sLbLinkConfig = '.' + cLbLinkConfig;
  var sLbSection = '.layout-builder__section';
  var sGsSection = sLbSection + '[' + _dgs + 'section]';
  var sGsLb = '.is-gs-lb';
  var sGsLbActivated = sGsLb + '.' + cLbHighligted;
  var sGridstackAction = '.' + cGridstackAction;
  var idLbSection = 'gslbsection';
  var idLbTopLink = 'onLbTopLink';
  var sLbTopLink = sLbLinkConfig + ':not(.' + cGsActionAjax + ')';
  var idVariantEdit = 'onVariantEdit';
  var sVariantEdit = '.btn--editor';
  var sContainer = '.is-gs-lb';
  var cGsSection = 'is-gs-section';
  var idVariantLabel = 'onVariantLabel';
  var sVariantLabel = '.form-text--label-variant';
  var sMainSettings = '.is-gs-main-settings';
  var sOffcanvas = '#drupal-off-canvas';
  var sOffcanvasForm = '#layout-builder-configure-section';
  var idOffcanvas = 'gsoffcanvas';
  var sOffcanvasSubmit = sOffcanvasForm + ' .js-form-submit';
  var dRegion = _dgs + 'region';
  var dGsSection = _dgs + 'section';
  var _ddsel = 'data-drupal-selector';
  var _attached = false;

  /**
   * Prepare region markers.
   *
   * @param {Event} e
   *   The mousedown event
   */
  function onLbTopLink(e) {
    gsCleanOut();
    gsSectionActivate(e);
  }

  /**
   * Cleans up region markers.
   *
   * @param {Event} e
   *   The mousedown event
   */
  function onModalClosed(e) {
    // dialog:afterclose is called after ML AJAX upload, too.
    // dialog:beforeclose is called when the form is not saved.
    setTimeout(function () {
      $.removeAttr(elBody, _dgs + 'variant-body');
      $.removeClass(elBody, 'is-gs-lb-saving');

      if (!$(sOffcanvas).length) {
        gsCleanOut();
      }
    }, 100);
  }

  // Reacts on dialog open.
  function onModalOpened(e, dialog, $element) {
    showActiveRegion($element, 'dialog:aftercreate');

    setTimeout(function () {
      var el = $element.length ? $element[0] : $element;
      if (Drupal.offCanvas &&
        Drupal.offCanvas.isOffCanvas($element) &&
        $.hasClass(_doc.body, 'is-gs-lb-saving')) {

        var elVid = $.find(el, '[' + _ddsel + '="edit-layout-settings-settings-global-vid"]');
        var elUpdate = $.find(el, '[' + _ddsel + '="edit-actions-submit"]');
        if ($.isElm(elUpdate)) {
          var vidAjax = $.attr(_doc.body, _dgs + 'variant-body');

          elVid.value = vidAjax;
          $.trigger(elUpdate, 'mousedown');
        }
      }

    }, 100);
  }

  /**
   * Cleans up region markers.
   */
  function gsCleanOut() {
    var elSection = $(sGsSection)[0];
    var $gsRegionActive = $('.' + cGsRegionActive);

    $.removeAttr(elBody, dRegion);

    if ($.isElm(elSection)) {
      $.removeAttr(elSection, dGsSection);
    }

    if ($gsRegionActive.length) {
      $gsRegionActive.removeClass(cGsRegionActive).attr('id', '');

      $gsRegionActive.each(function (item) {
        $(item).removeClass(cGsRegionActive).attr('id', '');
      });
    }

    $('.' + cFormSubmitActive).removeClass(cFormSubmitActive);
  }

  /**
   * Prepares region markers.

   * @param {Event} e
   *   The click/ mousedown event.
   */
  function gsSectionActivate(e) {
    var elSection = $(sGsSection)[0];
    var elGridstack = $(sGsLbActivated)[0];
    var link = e.target;

    if (!$.isElm(elSection)) {
      elSection = $.closest(link, sLbSection);
    }

    if (!$.isElm(elSection) && $.isElm(elGridstack)) {
      elSection = $.closest(elGridstack, sLbSection);
    }

    if ($.isElm(elSection) && $.isElm(elGridstack)) {
      $.attr(elBody, dRegion, true);
      $.attr(elSection, dGsSection, true);
    }
  }

  /**
   * Reacts on clicking gridstack__action--ajax-styles link.
   *
   * @param {Event} e
   *   The mousedown event.
   */
  function gsRegionActivate(e) {
    var link = e.target;
    var regionId = link.parentNode.dataset.gsRegion;

    gsCleanOut();
    gsSectionActivate(e);

    var box = $.closest(link, '.gridstack__box');
    $.addClass(box, cGsRegionActive);
    $.attr(box, 'id', cGsRegionActive);
    $.attr(elBody, dRegion, regionId);
  }

  /**
   * GridStack functions.
   *
   * @param {HTMLElement} $modal
   *   The modal HTML element.
   * @param {String} type
   *   The current event type.
   */
  function showActiveRegion($modal, type) {
    var activeRid = $(elBody).attr(dRegion);

    if (!$.isUnd(activeRid)) {
      var activeRegion = '[' + dRegion + '="' + activeRid + '"]';
      var elActiveRegion = $modal.find(activeRegion);

      if ($.isElm(elActiveRegion)) {
        var $activeRegion = $(elActiveRegion);
        var container = elActiveRegion.dataset.gsRegionContainer;
        var elContainer = $modal.find('[' + dRegion + '="' + container + '"]');
        var isCn = $activeRegion.hasClass('form-wrapper--cn');

        $activeRegion.attr('open', true).removeClass(cVisuallyHidden);
        if (isCn) {
          $activeRegion.removeAttr('open');
        }

        $(sMainSettings, $modal).removeAttr('open');
        var regions = $modal.findAll('[' + dRegion + ']:not(' + activeRegion + ', ' + sMainSettings + ')');

        if (regions.length) {
          $.each(regions, function (region) {
            $.addClass(region, cVisuallyHidden);
          });
        }

        var elAside = $modal.find('.is-gs-aside');
        $.addClass(elAside, cVisuallyHidden);

        if ($.isElm(elContainer)) {
          $.removeAttr(elContainer, 'open');
          $.removeClass(elContainer, cVisuallyHidden);
        }
      }
    }
  }

  /**
   * Reacts on button `Edit variant` AJAX events.
   *
   * @param {Event} e
   *   The event triggered by a `mousedown` event.
   */
  function gsOffcanvasSubmit(e) {
    var dialog = $.find(_doc, '.ui-dialog-off-canvas');
    if ($.isElm(dialog)) {
      $.addClass(dialog, 'is-b-loading');
    }
  }

  /**
   * Reacts on button `Edit variant` AJAX events.
   *
   * @param {Event} e
   *   The event triggered by a `mousedown` event.
   */
  function onVariantEdit(e) {
    var btn = e.target;
    var el = $.closest(btn, sLbSection);

    $.addClass(el, 'is-gs-editor-active');
  }

  /**
   * GridStack functions.
   *
   * @param {HTMLElement} lb
   *   The #layout-builder HTML element.
   */
  function gsContainer(lb) {
    var container = $.find(lb, sContainer);

    if (!$.isElm(container)) {
      return;
    }

    $.addClass(lb, cGsSection);

    var elLbSection = $.closest(container, sLbSection);
    var elLbConfigLink = $.find(elLbSection, '>' + sLbLinkConfig);

    // In case we have another wrapper (.gridstack-wrapper) above .gridstack.
    // We didn't hard-code class name previously since they might change.
    // This time is a willy-nilly. Also to support LB_UX.
    if (!$.isElm(elLbConfigLink)) {
      // layout-builder__actions.
      elLbConfigLink = $.find(elLbSection, '> div:first-child ' + sLbLinkConfig);
    }

    /**
     * Reacts on clicking non use-ajax link, fallback to AJAX.
     *
     * @param {Event} e
     *   The click event.
     */
    function gsActionJs(e) {
      e.preventDefault();

      var link = e.target;
      var cn = $.closest(link, sGridstackAction);
      var isOffCanvas = $(sOffcanvas).length;
      var $form;
      var elSection;
      var update = false;
      var ajax = true;

      if (isOffCanvas) {
        $.removeClass(cn, cLoading);

        elSection = $.closest(link, sGsSection);

        // Clicking inside active section, show region quickly, no need an AJAX.
        if ($.isElm(elSection)) {
          ajax = false;
          update = true;
        }
        // Clicking outside active section, trying to edit cross-section.
        else {
          update = $.isElm(elLbConfigLink);
        }
      }
      else {
        $.addClass(cn, cLoading);
        setTimeout(function () {
          $.removeClass(cn, cLoading);
        }, 5000);

        update = $.isElm(elLbConfigLink);
      }

      gsRegionActivate(e);
      if (update) {
        if ($.isElm(elLbConfigLink) && ajax) {
          elLbConfigLink.href = link.href;
          elLbConfigLink.click();
        }

        setTimeout(function () {
          $form = $(sOffcanvasForm);
          if ($form.length) {
            $form.addClass('is-gs-form');
            showActiveRegion($form, 'click');
          }
        }, isOffCanvas ? 0 : 5000);
      }
    }

    /**
     * GridStack functions.
     *
     * @param {HTMLElement} elm
     *   The .gridstack__action HTML element.
     */
    function gsActionItem(elm) {
      var $elm = $(elm);
      var regionId = $elm.attr(dRegion);

      var addAttributes = function (link) {
        var href = link.pathname;
        if (href && !$.contains(href, 'gs=')) {
          var part = $.contains(href, '?') ? '&gs=' : '?gs=';
          href = href + part + regionId;
          $.attr(link, 'href', href);
          $.attr(link, dRegion, regionId);
          link.textContent = Drupal.t('Styles');
        }
      };

      if ($.isElm(elLbConfigLink)) {
        var cloned = elLbConfigLink;
        var elLinkJs = $.find(elm, '.' + cGsActionJs);

        if (!$.isElm(elLinkJs)) {
          var elClone2 = cloned.cloneNode(false);

          $.addClass(elClone2, 'layout-builder__link--add ' + cGsActionJs);
          $.removeClass(elClone2, 'use-ajax');
          $.removeAttr(elClone2, 'data-once data-dialog-type data-dialog-renderer');

          // Drupal.detachBehaviors(elClone2);
          elm.appendChild(elClone2);
        }

        elLinkJs = $.find(elm, '.' + cGsActionJs);
        addAttributes(elLinkJs);

        $.on(lb, 'click.gsActionJs', '.' + cGsActionJs, gsActionJs, false);
      }
    }

    /**
     * Provides inline styles.
     */
    function gsInlineStyles() {
      var stylesId;
      var curStyles;
      var _prefix = 'gridstack--style-';
      var config = container.dataset.gsInlineStyles || {};

      function initStyles() {
        var opts = {
          id: stylesId,
          prefix: _prefix
        };
        var gridStyle = GridStyle.init(opts, container);

        stylesId = gridStyle.id;
        curStyles = gridStyle.styles;
      }

      function updateStyles() {
        if ($.isNull(curStyles) || $.isUnd(curStyles)) {
          initStyles();
        }

        if ($.isNull(curStyles) || $.isUnd(curStyles)) {
          return;
        }

        initStyles();

        if (config) {
          var i = 0;
          $.each(config, function (rule, selector) {
            GridStyle.insertCSSRule(curStyles, selector, rule, i);
            i++;
          });
        }
      }

      updateStyles();
    }

    gsInlineStyles();

    //  + ':not(.' + cGridstackAction + '--on)'
    var actions = $.findAll(container, sGridstackAction);
    if (actions.length) {
      $.each(actions, gsActionItem);
    }

    _attached = true;
  }

  /**
   * Reacts on blur event.
   *
   * @param {Event} e
   *   The blur event.
   */
  function onVariantLabel(e) {
    var input = e.target;
    var value = input.value;
    var form = $.closest(input, '.form--gridstack-variant');
    var btn = $.find(form, 'a.btn--editor-duplicate');

    if ($.isElm(btn)) {
      var href = btn.href;

      if ($.contains(href, 'label')) {
        href = href.substring(0, href.indexOf('&label='));
      }

      href = href + '&label=' + Drupal.checkPlain(value);
      $.attr(btn, 'href', href);
    }
  }

  /**
   * Attaches gridstack behavior to HTML element .gridstack.is-gs-lb.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.gridStackAdminLayout = {
    attach: function (context) {

      context = $.context(context);

      elBody = _doc.body;
      var $body = $(elBody);

      $body.off('mousedown.' + idLbTopLink).on('mousedown.' + idLbTopLink, sLbTopLink, onLbTopLink);
      $body.off('mousedown.' + idVariantEdit).on('mousedown.' + idVariantEdit, sVariantEdit, onVariantEdit);
      $body.off('blur.' + idVariantLabel).on('blur.' + idVariantLabel, sVariantLabel, Drupal.debounce(onVariantLabel, 200));
      $(sOffcanvasSubmit).off('mousedown.' + idOffcanvas).on('mousedown.' + idOffcanvas, gsOffcanvasSubmit);

      $.once(gsContainer, idLbSection, sLbSection, context);
    },
    detach: function (context, settings, trigger) {
      if (trigger === 'unload') {
        if (_attached) {
          context = $.context(context);
          var $body = $(elBody);
          $body.off('mousedown.' + idLbTopLink);
          $body.off('mousedown.' + idVariantEdit);
          $body.off('blur.' + idVariantLabel);

          $(sOffcanvasSubmit).off('mousedown.' + idOffcanvas);

          $.once.removeSafely(idLbSection, sLbSection, context);
        }
      }
    }
  };

  // @todo update when core removes jQuery UI dialog, likey D10+.
  // Drupal off-canvas register these events as jQuery object, not native.
  if (jQuery) {
    jQuery(_win).on({
      'dialog:afterclose': onModalClosed,
      'dialog:aftercreate': onModalOpened
    });
  }

})(dBlazy, Drupal, GridStyle, this, this.document);
